class Subdomain(object):
    """ Represent a subdomain tuple """
    def __init__(self, domain, typ, ip):
        self.domain = domain
        self.typ = typ
        self.ip = ip

    def __repr__(self):
        return f'<Subdomain domain="{self.domain}", type="{self.typ}", ip="{self.ip}">'

    @staticmethod
    def from_string(some_string):
        parts = some_string.split()
        if len(parts) != 3:
            raise ValueError('The string should have three parts')
        
        return Subdomain(parts[0], parts[1], parts[2])


from pathlib import Path
from datetime import datetime
import requests
import re

_cert_sh_domain_regex = re.compile(r'<td>([\d\w\-\.]*?)<\/td>', re.I)

def convert_to_path(some_path):
    """ Safely converts a string like path, to a pathlib variant """
    if isinstance(some_path, str):
        return Path(some_path)
    elif isinstance(some_path, Path):
        return some_path
    else:
        raise ValueError('Can\'t conver to Path')

def generate_subdomains(fname: Path, domain: str):
    """ Reads subdomin prefixes - lazy """

    with fname.open('r') as f:
        for line in f:
            sub_domain = line.strip()
            yield f'{sub_domain}.{domain}'

def now_in_str():
    """ Gets todays date in a string form """
    return datetime.now().strftime('%Y%m%d_%H%M%S')


def get_crt_sh_subdomains(domain: str) -> list:
    """ Extracts subdomains from cert.sh """
    def _find_domain_in_body(html_body: str) -> set:
        domains = set()

        for match in _cert_sh_domain_regex.finditer(html_body):
            domains.add(match[1].lower())

        return domains

    url = f'https://crt.sh/?q=%25{domain}'
    r = requests.get(url)

    return _find_domain_in_body(r.text)


from pathlib import Path
from subprocess import Popen, PIPE
from itertools import chain

from massdns.utils import convert_to_path, now_in_str, generate_subdomains, get_crt_sh_subdomains
from massdns.subdomain import Subdomain


class MassDNS(object):
    """ Represents the MassDNS tool wrapper class"""

    def __init__(self, massdns_root_dir):
        self.root_dir = convert_to_path(massdns_root_dir)
        self.default_names_path = self.root_dir / 'lists' / 'names_small.txt'
        self.default_resolvers_path = self.root_dir / 'lists' / 'resolvers.txt'
        self.binary_dir = self.root_dir / 'bin' / 'massdns'
        self.results_folder = self.root_dir / 'results'

        self._create_results_folder()

    def _create_results_folder(self):
        if not self.results_folder.exists():
            self.results_folder.mkdir()


    def scan(self, domain: str, cert_sh=False, names_path=None, resolvers_path=None):
        names_path = convert_to_path(names_path or self.default_names_path)
        resolvers_path = convert_to_path(resolvers_path or self.default_resolvers_path)
        output_path = self.results_folder / f'{domain}_{now_in_str()}.txt'

        command = [
            self.binary_dir,
            '-s', '20000',
            '-r', resolvers_path,
            '-t', 'A',
            '-o', 'S',
            '-w', output_path
        ]

        process = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)

        subs = generate_subdomains(names_path, domain)

        if cert_sh:
            subs = chain(subs, get_crt_sh_subdomains(domain))

        process.stdin.write('\n'.join(subs).encode())

        _ = process.communicate()
        output_data = []

        with output_path.open('r') as f:
            return [Subdomain.from_string(line.strip()) for line in f]
    


import os
from setuptools import setup

from massdns import __version__
from massdns import __author__

setup(
    name = "pymassdns",
    version = __version__,
    author = __author__,
    author_email = "abeles22@gmail.com",
    description = "This is a python3 wrapper to MassDNS tool",
    license = "BSD",
    keywords = "massdns, bruteforce, dns, scanner",
    url='https://github.com/Den1al/pymassdns',
    packages=['massdns'],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Topic :: System :: Monitoring",
        "Topic :: System :: Networking",
        "Topic :: System :: Networking :: Firewalls",
        "Topic :: System :: Networking :: Monitoring",
    ],
    install_requires=[
          'requests',
      ],
)
